<?php

namespace FiloBlu\OrderStatus\Observer;

use \Magento\Framework\Event\ObserverInterface;
use \Magento\Framework\Event\Observer;
use \Magento\Framework\App\Config\ScopeConfigInterface;
use \Magento\Store\Model\ScopeInterface;

/**
 * Observer on Adyen notification and set order status accordingly
 * This observer needs the core patch adyen_notification_events.patch
 */

class UpdateStatusRefund implements ObserverInterface {

    /**
     * @var \Psr\Log\LoggerInterface $logger
     */
    protected $_logger;

    protected $_config;

    const MANAGED_NOTIFICATION_CODES = array('REFUND');
    const FAILED_REFUND_ORDER_STATUS = 'failed_refund';
    const DEBUG_LOG_ACTIVE = TRUE;

    /**
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        ScopeConfigInterface $config
    ){
        $this->_logger = $logger;
        $this->_config = $config;
    }

    /**
     * Process order
     *
     * @param \Magento\Framework\Event\Observer $observer
     */
    public function execute(Observer $observer) {
        /* @var $order \Magento\Sales\Model\Order */
        if (self::DEBUG_LOG_ACTIVE) {
            $this->_logger->addInfo("FiloBlu\OrderStatus\Observer: Processing Unsuccessful Notification");
        }
        $notification_code = $observer->getEvent()->getNotificationCode();
        if (!in_array($notification_code, self::MANAGED_NOTIFICATION_CODES)) {
            if (self::DEBUG_LOG_ACTIVE) {
                $this->_logger->addInfo("FiloBlu\OrderStatus\Observer: Skipped notification");
            }
            return;
        }

        $order = $observer->getEvent()->getOrder();
        $payment_method = $order->getPayment()->getMethod();
        $payment_method_type = $order->getPayment()->getPaymentMethodType();
        if (self::DEBUG_LOG_ACTIVE) {
            $this->_logger->addInfo("FiloBlu\OrderStatus\Observer: Processing Order " . $order->getIncrementId());
            $this->_logger->addInfo("FiloBlu\OrderStatus\Observer: payment_method: $payment_method");
            $this->_logger->addInfo("FiloBlu\OrderStatus\Observer: payment_method_type: $payment_method_type");
        }

        $order->setState(\Magento\Sales\Model\Order::STATE_CLOSED)
            ->addStatusHistoryComment(__('Order processed by mod2-order-status'), self::FAILED_REFUND_ORDER_STATUS);

        $order->save();

        return TRUE;
    }
}